<?php
/**
 * @file: TwigExtension.php
 * @author: Rene Bakx (rene@renebakx.nl)
 */

namespace Drupal\tlh;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\block\Entity\Block;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\Markup;

class TwigExtension extends \Twig_Extension implements \Twig_ExtensionInterface {

  public function getName() {
    return 'ThemersLittleHelper';
  }


  /**
   * The block builder
   * @var \Drupal\Core\Entity\EntityViewBuilderInterface
   */
  protected $blockBuilder;
  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;


  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * TwigExtension constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager The entity type manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory The configuration factory.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ConfigFactoryInterface $config_factory) {
    $this->blockBuilder = \Drupal::entityTypeManager()->getViewBuilder('block');
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
  }
  
  /**
   * {@inheritdoc}
   */
  public function getGlobals() {
    global $request, $base_insecure_url, $base_secure_url;
    $globals = [];
    $globals['base_url'] = $base_insecure_url;
    if ($request->isSecure()) {
      $globals['base_url'] = $base_secure_url;
    }
    return $globals;
  }
  /**
   * {@inheritdoc}
   */
  public function getFunctions() {

    $return =  [
      new \Twig_SimpleFunction('drupal_view', [$this, 'drupalView']),
      new \Twig_SimpleFunction('drupal_block', [$this, 'drupalBlock'])
    ];
    // hook in support for the Symfony vardumper
    if ( \Drupal::moduleHandler()->moduleExists('vardumper')){

      $return[] = new \Twig_SimpleFunction('vdp','vdp');
      $return[] = new \Twig_SimpleFunction('vdm','vdpm');
      $return[] = new \Twig_SimpleFunction('dump','dump');

    }
    return $return;
  }

  public function getFilters() {
    return [
      new \Twig_SimpleFilter('wrap', [$this, 'wrapTag'], [
        'is_safe' => ['html'],
      ]),

    ];
  }

  /**
   * Return the render array for a view
   * @param $name
   * @param string $display_id
   * @return mixed
   */
  public function drupalView($name, $display_id = 'default') {
    $args = func_get_args();
    return views_embed_view(
      $name,
      $display_id,
      isset($args[2]) ? $args[2] : NULL,
      isset($args[3]) ? $args[3] : NULL,
      isset($args[4]) ? $args[2] : NULL
    );
  }


  /**
   * Returns the render array for a block by $ID
   * @param $id
   * @return array
   */
  public function drupalBlock($id) {
    $block = Block::load($id);
    return $this->entityTypeManager->getViewBuilder('block')->view($block);
  }


  /**
   * Wraps the $context with the given tag
   *
   * @param $value
   * @param $tag
   * @return mixed
   */
  function wrapTag($value, $tag) {
    if (isset($value['#theme'])) {
      $value = \Drupal::service('renderer')->render($value);
      if ($value instanceof Markup) {
        $value = $value->__toString();
      }
    }

    if (is_scalar($value)) {
      return sprintf('<%s>%s</%s>', $tag, trim($value), $tag);
    }
    return $value;
  }
}
